//
//  HelloLiveCameraViewController.m
//  HelloLiveCamera
//
//  Created by Douglass Turner on 12/13/11.
//  Copyright (c) 2011 __MyCompanyName__. All rights reserved.
//

#import "HelloLiveCameraViewController.h"
#import "Logging.h"

@implementation HelloLiveCameraViewController

@synthesize captureSession;
@synthesize customLayer;

- (void)dealloc {
    
    self.captureSession = nil;
    self.customLayer = nil;
    
    [super dealloc];
}

- (void)viewDidLoad {
    [super viewDidLoad];
    
	[self initCapture];
    
}

- (void)viewDidUnload {
    [super viewDidUnload];

}

- (void)initCapture {
    
    // Capture input
	AVCaptureDeviceInput *captureInput = [AVCaptureDeviceInput deviceInputWithDevice:[AVCaptureDevice defaultDeviceWithMediaType:AVMediaTypeVideo] error:nil];
    
    // Capture output
	AVCaptureVideoDataOutput *captureOutput = [[[AVCaptureVideoDataOutput alloc] init] autorelease];
    
	/*While a frame is processes in -captureOutput:didOutputSampleBuffer:fromConnection: delegate methods no other frames are added in the queue.
	 If you don't want this behaviour set the property to NO */
	captureOutput.alwaysDiscardsLateVideoFrames = YES; 
    
	/*We specify a minimum duration for each frame (play with this settings to avoid having too many frames waiting
	 in the queue because it can cause memory issues). It is similar to the inverse of the maximum framerate.
	 In this example we set a min frame duration of 1/10 seconds so a maximum framerate of 10fps. We say that
	 we are not able to process more than 10 frames per second.*/
	//captureOutput.minFrameDuration = CMTimeMake(1, 10);
    
    // Set the video output to store frame in BGRA (It is supposed to be faster)
	[captureOutput setVideoSettings:[NSDictionary dictionaryWithObject:[NSNumber numberWithUnsignedInt:kCVPixelFormatType_32BGRA] forKey:(NSString *)kCVPixelBufferPixelFormatTypeKey]]; 
    
    // Create serial queue to process frames
	dispatch_queue_t queue;
	queue = dispatch_queue_create("cameraQueue", NULL);
	[captureOutput setSampleBufferDelegate:self queue:queue];
	dispatch_release(queue);
    
    
    // Create capture session. Add input & output
	self.captureSession = [[[AVCaptureSession alloc] init] autorelease];
	[self.captureSession addInput:captureInput];
	[self.captureSession addOutput:captureOutput];
    
    // Use medium quality on iPhone4 for this demo otherwise the demo would lag too much. 
    // Conversion in UIImage and CGImage demands too much ressources for 720p resolution
    //    [self.captureSession setSessionPreset:AVCaptureSessionPresetLow];
    [self.captureSession setSessionPreset:AVCaptureSessionPresetMedium];
    //    [self.captureSession setSessionPreset:AVCaptureSessionPresetHigh];
    
    
    // Add custom layer. Change orientation for correct display.
	self.customLayer = [CALayer layer];
	self.customLayer.frame = self.view.bounds;
	self.customLayer.transform = CATransform3DRotate(CATransform3DIdentity, M_PI/2.0f, 0, 0, 1);
	self.customLayer.contentsGravity = kCAGravityResizeAspectFill;
	[self.view.layer addSublayer:self.customLayer];
    
    // Go for it.
	[self.captureSession startRunning];
	
}

- (void)captureOutput:(AVCaptureOutput *)captureOutput didOutputSampleBuffer:(CMSampleBufferRef)sampleBuffer fromConnection:(AVCaptureConnection *)connection { 
	
    DLog(@"");
    
	NSAutoreleasePool * pool = [[NSAutoreleasePool alloc] init];
	
    // Get and lock image buffer
    CVImageBufferRef imageBuffer = CMSampleBufferGetImageBuffer(sampleBuffer); 
    CVPixelBufferLockBaseAddress(imageBuffer,0); 
    
    /*Get information about the image*/
    uint8_t *baseAddress = (uint8_t *)CVPixelBufferGetBaseAddress(imageBuffer); 
    size_t bytesPerRow = CVPixelBufferGetBytesPerRow(imageBuffer); 
    size_t width = CVPixelBufferGetWidth(imageBuffer); 
    size_t height = CVPixelBufferGetHeight(imageBuffer);  
    
    /*Create a CGImageRef from the CVImageBufferRef*/
    CGColorSpaceRef colorSpace = CGColorSpaceCreateDeviceRGB(); 
    CGContextRef _context = CGBitmapContextCreate(baseAddress, width, height, 8, bytesPerRow, colorSpace, kCGBitmapByteOrder32Little | kCGImageAlphaPremultipliedFirst);
    CGImageRef newImage = CGBitmapContextCreateImage(_context); 
	
    /*We release some components*/
    CGContextRelease(_context); 
    CGColorSpaceRelease(colorSpace);
    
    /*We display the result on the custom layer. All the display stuff must be done in the main thread because
	 UIKit is no thread safe, and as we are not in the main thread (remember we didn't use the main_queue)
	 we use performSelectorOnMainThread to call our CALayer and tell it to display the CGImage.*/
	[self.customLayer performSelectorOnMainThread:@selector(setContents:) withObject: (id) newImage waitUntilDone:YES];
	
	CGImageRelease(newImage);
	
	/*We unlock the  image buffer*/
	CVPixelBufferUnlockBaseAddress(imageBuffer,0);
	
	[pool drain];
} 

- (NSString *)nibName {
    
    if ([[UIDevice currentDevice] userInterfaceIdiom] == UIUserInterfaceIdiomPhone) {
        
        return @"HelloLiveCameraViewController_iPhone";
        
    } else {
        
        return @"HelloLiveCameraViewController_iPad";
        
    }

}

@end